/*
   So far we have a "tree language" for describing binary trees,
   and a "parser" that can convert strings from that language
   into Java binary tree data structures. We also have a "binary
   arithmetic language" that is a subset of the binary tree language.

   If we take a binary arithmetic expression like
       3 * 4 + 5 + 6
   and write it in its "prefix" notation
      (+ ( + ( * 3 4 ) 5 ) 6 )
   we now have a string in our binary arithmetic language
   (and also a string  in our binary tree language), so we
   can use our parser to build the "abstract syntax tree" for
   this expression. Once we have the AST for the expression,
   we can evaluate it using a simple post-order traversal of
   the AST.
*/

public class Language_1_Example_2
{
   public static void main(String[] args)
   {
      String[] treeStrings = {
                               "( + ( + ( * 3 4 ) 5 ) 6 )", // 3*4+5+6
                               "( * 3 ( + ( + 4 5 ) 6 ) )", // 3*(4+5+6)
                               "( + 3 ( + 4 ( + 5 6 ) ) )", // 3+(4+(5+6))
                               "( + ( + 3 4 ) ( * 5 6 ) )"  // 3+4+5*6
                             };
      int i = 0;
      for (i = 0; i < treeStrings.length; i++)
      {
         System.out.println(i + " =============================================");

         // Print the expression.
         System.out.println( " Original expression = " + treeStrings[i] + "\n" );

         // Build the BTree data structure from its string representation.
         BTree btree = BuildBTree.buildBTree( treeStrings[i] );

         // Pretty-print the btree.
         System.out.println( PrettyPrinter.prettyPrint( btree ) + "\n" );

         // Print the infix version of the expression.
         System.out.println( "    Infix expression = " + AST2infix.ast2infix(  btree ) + "\n" );
         System.out.println( "    Infix expression = " + AST2infix2.ast2infix( btree ) + "\n" );

         // Print the three tree traversals.
         System.out.println( "  In-order traversal = " + Traverse.inOrder(   btree ) + "\n" );
         System.out.println( " Pre-order traversal = " + Traverse.preOrder(  btree ) + "\n" );
         System.out.println( "Post-order traversal = " + Traverse.postOrder( btree ) + "\n" );

         // Evaluate the expression tree.
         int result = Evaluate.eval( btree );
         System.out.println( treeStrings[i] + " ==> " + result + "\n" );


         // Create dot and png files from the btree.
         if(true)
         try
         {
            // Create the (empty) dot file.
            String baseName = String.format("Example_2.%02d", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot"));
            // Put dot commands into the dot file
            out.println( BTree2dot_ver1.btree2dot(btree) + "\n" );
            out.close();

            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()
}
